%let cty =  AUS AUT BFL BFR CAN COL CYP CZE DNK ENG
            FRA DEU GRC HKG HUN ISL IRN IRL ISR JPN
            KOR KWT LVA LTU NLD NZL NOR PRT ROM RUS
            SCO SGP SVK SVN ESP SWE CHE THA USA ITA ;


Options nosource nocenter;

LIBNAME libdat  "<path>" ;
LIBNAME library "<path>" ;

%macro doit;
  %let i = 1;
     %do %while(%length(%scan(&cty,&i)));
       %let ccode = %lowcase(%scan(&cty,&i));

%let fname= bst&ccode.m1 ;

FILENAME rawinp "<path>&fname..dat" ;

%PUT INFO: data set definition ;
%PUT INFO: data set definitions may need to be modified ;

*Output data set ;
data libdat.&fname;

*NOTE: LRECL may need to be modified ;
INFILE rawinp LRECL=1100 END=eof LENGTH=lnline LINE=ln MISSOVER ;

%PUT INFO: shorten variable definitions to save secondary storage area ;
%PUT INFO: refer to the SAS manual for an appropriate length for variables ;

*NOTE: The LENGTH statement in a data step specifies the number of bytes, the
       SAS - sytem uses for storing values of variables in each data set being createt. ;
LENGTH DEFAULT=5 ;
LENGTH      VERSION    3 ;
LENGTH      IDCNTRY    5 ;
LENGTH      IDPOP      3 ;
LENGTH      IDSCHOOL   6 ;
LENGTH      IDCLASS    8 ;
LENGTH      IDSTUD     8 ;
LENGTH      IDTEACH    8 ;
LENGTH      IDLINK     3 ;
LENGTH      IDGRADER   3 ;
LENGTH      INTMS95    3 ;
LENGTH      JKZONE     3 ;
LENGTH      JKREP      3 ;
LENGTH      MATSUBJ    3 ;
LENGTH      SCISUBJ    3 ;
LENGTH      NMTEACH    3 ;
LENGTH      NSTEACH    3 ;
LENGTH      NTEACH     3 ;
LENGTH      MATWGT     8 ;
LENGTH      SCIWGT     8 ;
LENGTH      TCHWGT     8 ;
LENGTH      TOTWGT     8 ;
LENGTH      BSMMAT01   8 ;
LENGTH      BSMMAT02   8 ;
LENGTH      BSMMAT03   8 ;
LENGTH      BSMMAT04   8 ;
LENGTH      BSMMAT05   8 ;
LENGTH      BSSSCI01   8 ;
LENGTH      BSSSCI02   8 ;
LENGTH      BSSSCI03   8 ;
LENGTH      BSSSCI04   8 ;
LENGTH      BSSSCI05   8 ;
LENGTH      DPCDATE    6 ;


%PUT INFO: Missing category A refers to "not administered" values ;
%PUT INFO: Missing category I refers to "invalid" values ;
%PUT INFO: Missing category B refers to "log. not applicable" values ;
%PUT INFO: Missing category B refers to "not reached" items ;
%PUT INFO: Missing category N refers to "non sense" answers ;
%PUT INFO: Missing category . refers to missing values ;
MISSING A I B R N ;

*NOTE: The INPUT statement reads raw data and assigns variable names.
       The positions specified below correspond to the positions of the variables
       in the raw data file. ;

INPUT
  VERSION       1-   2     
  IDCNTRY       3-   7     
  IDPOP         8-   8     
  IDSCHOOL      9-  14     
  IDCLASS      15-  22     
  IDSTUD       23-  32     
  IDTEACH      33-  40     
  IDLINK       41-  42     
  IDGRADER     43-  43     
  INTMS95      44-  44     
  JKZONE       45-  46     
  JKREP        47-  47     
  MATSUBJ      48-  48     
  SCISUBJ      49-  49     
  NMTEACH      50-  50     
  NSTEACH      51-  51     
  NTEACH       52-  52     
  MATWGT       53-  65 .6
  SCIWGT       66-  78 .6
  TCHWGT       79-  91 .6
  TOTWGT       92- 104 .6
  BSMMAT01    105- 110 .2
  BSMMAT02    111- 116 .2
  BSMMAT03    117- 122 .2
  BSMMAT04    123- 128 .2
  BSMMAT05    129- 134 .2
  BSSSCI01    135- 140 .2
  BSSSCI02    141- 146 .2
  BSSSCI03    147- 152 .2
  BSSSCI04    153- 158 .2
  BSSSCI05    159- 164 .2
  DPCDATE     165- 170     
;


%PUT INFO: assign labels to variables ;
LABEL
  VERSION  = "*FILE VERSION*"
  IDCNTRY  = "*COUNTRY ID*"
  IDPOP    = "*POPULATION ID*"
  IDSCHOOL = "*SCHOOL ID*"
  IDCLASS  = "*CLASS ID*"
  IDSTUD   = "*STUDENT ID*"
  IDTEACH  = "*TEACHER ID*"
  IDLINK   = "*TEACHER LINK NUMBER*"
  IDGRADER = "*GRADE*"
  INTMS95  = "*INTMS95*"
  JKZONE   = "*ZONE CODE*"
  JKREP    = "*REPLICATE CODE*"
  MATSUBJ  = "*MATHEMATICS TEACHER LINK*"
  SCISUBJ  = "*Science Teacher Link*"
  NMTEACH  = "*Number of Math Teachers for Student*"
  NSTEACH  = "*Number of Science Teachers for Student*"
  NTEACH   = "*# of Math & Science Teachers combined*"
  MATWGT   = "*Weight for Mathematics Teacher Data*"
  SCIWGT   = "*Weight for Science Teacher Data*"
  TCHWGT   = "*Weight for Mat & Sci Teacher Data combin*"
  TOTWGT   = "*STUDENT WEIGHT*"
  BSMMAT01 = "*1ST PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT02 = "*2ND PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT03 = "*3RD PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT04 = "*4TH PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT05 = "*5TH PLAUSIBLE VALUE MATHEMATICS*"
  BSSSCI01 = "*1ST PLAUSIBLE VALUE SCIENCE*"
  BSSSCI02 = "*2ND PLAUSIBLE VALUE SCIENCE*"
  BSSSCI03 = "*3RD PLAUSIBLE VALUE SCIENCE*"
  BSSSCI04 = "*4TH PLAUSIBLE VALUE SCIENCE*"
  BSSSCI05 = "*5TH PLAUSIBLE VALUE SCIENCE*"
  DPCDATE  = "*FILE CREATION DATE AT DPC HAMBURG*"
;

%PUT INFO: recode "not administered" data to missing code ".A" ;
*NOTE: in some cases a zero code might be appropriate for numeric values
       in this case the variables must be recoded correspondingly. This is
       not done by this program.
       If a CHECK variable is coded correctly with a BLANK, it will be recoded
       to "1", otherwise it will be recoded to "0" ;

IF VERSION  = 8                THEN VERSION  = .A ;
IF IDCNTRY  = 99998            THEN IDCNTRY  = .A ;
IF IDPOP    = 8                THEN IDPOP    = .A ;
IF IDSCHOOL = 999998           THEN IDSCHOOL = .A ;
IF IDCLASS  = 99999998         THEN IDCLASS  = .A ;
IF IDGRADER = 8                THEN IDGRADER = .A ;
IF INTMS95  = 8                THEN INTMS95  = .A ;
IF JKZONE   = 98               THEN JKZONE   = .A ;
IF JKREP    = 8                THEN JKREP    = .A ;
IF MATWGT   = 999998           THEN MATWGT   = .A ;
IF SCIWGT   = 999998           THEN SCIWGT   = .A ;
IF TCHWGT   = 999998           THEN TCHWGT   = .A ;
IF TOTWGT   = 999998           THEN TOTWGT   = .A ;
IF BSMMAT01 = 998              THEN BSMMAT01 = .A ;
IF BSMMAT02 = 998              THEN BSMMAT02 = .A ;
IF BSMMAT03 = 998              THEN BSMMAT03 = .A ;
IF BSMMAT04 = 998              THEN BSMMAT04 = .A ;
IF BSMMAT05 = 998              THEN BSMMAT05 = .A ;
IF BSSSCI01 = 998              THEN BSSSCI01 = .A ;
IF BSSSCI02 = 998              THEN BSSSCI02 = .A ;
IF BSSSCI03 = 998              THEN BSSSCI03 = .A ;
IF BSSSCI04 = 998              THEN BSSSCI04 = .A ;
IF BSSSCI05 = 998              THEN BSSSCI05 = .A ;

%PUT INFO: recode special missing codes to SAS missing codes ;
*NOTE: Special missing codes are recoded to SAS
       character - missing codes


%PUT INFO: recode missing data to missing code "." ;
*NOTE: BLANKS are recoded to SYSMIS by SAS ;

IF VERSION  = 9                THEN VERSION  = . ;
IF IDCNTRY  = 99999            THEN IDCNTRY  = . ;
IF IDPOP    = 9                THEN IDPOP    = . ;
IF IDSCHOOL = 999999           THEN IDSCHOOL = . ;
IF IDCLASS  = 99999999         THEN IDCLASS  = . ;
IF IDGRADER = 9                THEN IDGRADER = . ;
IF INTMS95  = 9                THEN INTMS95  = . ;
IF JKZONE   = 99               THEN JKZONE   = . ;
IF JKREP    = 9                THEN JKREP    = . ;
IF MATWGT   = 999999           THEN MATWGT   = . ;
IF SCIWGT   = 999999           THEN SCIWGT   = . ;
IF TCHWGT   = 999999           THEN TCHWGT   = . ;
IF TOTWGT   = 999999           THEN TOTWGT   = . ;
IF BSMMAT01 = 999              THEN BSMMAT01 = . ;
IF BSMMAT02 = 999              THEN BSMMAT02 = . ;
IF BSMMAT03 = 999              THEN BSMMAT03 = . ;
IF BSMMAT04 = 999              THEN BSMMAT04 = . ;
IF BSMMAT05 = 999              THEN BSMMAT05 = . ;
IF BSSSCI01 = 999              THEN BSSSCI01 = . ;
IF BSSSCI02 = 999              THEN BSSSCI02 = . ;
IF BSSSCI03 = 999              THEN BSSSCI03 = . ;
IF BSSSCI04 = 999              THEN BSSSCI04 = . ;
IF BSSSCI05 = 999              THEN BSSSCI05 = . ;

%PUT INFO: assign variable labels to recoded variables ;
LABEL
;

RUN ;

proc sort ; by idcntry idteach idlink idstud;

       %let i = %eval(&i + 1);
     %end;

%mend doit;

%doit;










